#BitScope PiLab
An open source science lab for [Raspberry Pi](http://raspberrypi.org/) and [BitScope](http://bitscope.com/).

**PiLab** is an easy to use open source programmable platform written in standard Python for use with Raspberry Pi. It's designed to enable the creation of custom test, measurement and data acquisition applications in educational, engineering and scientific fields.  

It uses a standalone PNG based graphical UI system that requires no complicated widget libraries or it can be used from the command line without requiring any graphics.

**PiLab** has few system prerequisites:

1. [Python 2.7](https://www.python.org/) or later for PiLab programming,
2. [PyGame](http://www.pygame.org/) modules (for SDL based graphics), and
3. [PySerial](http://pyserial.sourceforge.net/) for USB access to the BitScope.

All these are installed on Raspberry Pi as part of the default Raspbian distribution but PySerial may require upgrading to the latest version if you're running an older Raspbian release:

    $ sudo apt-get install python-pip
    $ sudo pip install pyserial --upgrade

Of course you'll also need a BitScope ([Micro](http://bitscope.com/product/BS05/) or [Model 10](http://bitscope.com/product/BS10/)) and a Raspberry Pi (Model B or B+) with a spare USB port (Model B+ has plenty!) capable of delivering 100mA of power.

##Quick Start

The **PiLab** code is *pure Python* so you simply need to run it *in situ*.

 1. Make sure your BitScope ([BS10][1], or [BS05][2]) is connected.
 1. Change directory to the root folder of the respository in a terminal.
 1. In your terminal window type: **python main.py**

PiLab should start by opening a 1280x720 window with selection buttons down the left side to start one of the built-in PiLab applications. 

PiLab comes with a range of examples to get you started but you can create your own by cloning and modifying one of these and edit its JSON configuration files, PNG image files and (optionally) Python source to create custom applications of your own.

##BitScope Micro Test & Verify

What better example of PiLab than to use it to build an application to test a BitScope!

Manufacture Q/A testing is vital to ensure a product works and meets its design specifications so we thought we'd use PiLab to build a highly optimised Q/A Test and Verification system to test the operation of [BitScope Micro](http://bitscope.com/product/BS05/) in production. 

Such an application needs to be reliable, easy to use and quick. Two of the example apps we've included in PiLab are the Q/A applications: **Test** and **Verify**. We use modified versions of these apps in the manufacture of BitScope Micro itself. They rely on a passive **test circuit** which you can build from a few resistors and capacitors (see **here** for details).

We'll add more examples and we encourage you to create your own!

##Project Information and Status
BitScope PiLab is a complete and operational platform but its documentation is in need of some love. The Python source is reasonably self-explanatory and the JSON files are easy to read (if rather deeply nested at times) and modify. Amending the Python source requires a detailed knowledge of BitScope Virtual Machine (BVM) programming. This is not hard but we recommend you read the **BVM Programming Guide** and some **BVM Tutorials** to learn how it works before trying to make wholesale changes to the Python source of a PiLab app. 

  [1]: http://bitscope.com/product/BS10/
  [2]: http://bitscope.com/product/BS05/