import random
import serial
from serial_tools import Serial_Tools
from tools import *
from volt_m_tools import *
import time

from scope_commander import Scope_Commander

class Volt_M(object):
    def __init__(self):        
        """Serial and other"""
        self.model = ""
        self.device_connected = False
        self.ser = None


        self.s_t = Serial_Tools(self.ser) #Tool to help issuing without waiting.
        self.s_t.show_messages = False
        
        scom = Scope_Commander(self.ser)

        """TRACE AND DUMP SETTINGS"""
        self.trace_size = 512 #captured trace size in samples.
        self.extra_trace = 16 #ADC warmup junk
        self.whole_trace = self.trace_size + self.extra_trace
        self.d_dec = 16
        self.d_size = (self.trace_size / self.d_dec) * 2 #dump size in bytes
        self.ex_size = (self.extra_trace / self.d_dec) * 2 #extra samples for ADC warmup in bytes
        self.whole_dump = ((self.trace_size + self.extra_trace) / self.d_dec) * 2

        self.div = float(self.d_size)

        self.data = {}
        self.data['ranges'] = {}
        self.data['ranges']['BS0005'] = {
            1.1 :           "[64]@[56]sn[7e]sn[77]sn[82]s",
            3.5 :           "[64]@[c3]sn[6a]sn[a3]sn[95]s",
            5.2 :           "[64]@[68]sn[44]sn[ff]sn[8a]s",
            11 :            "[64]@[6a]sn[12]sn[8c]sn[ba]s",
            "better 11" :   "[64]@[6f]sn[14]sn[8c]sn[ba]s",
            "max" :         "[64]@[00]sn[00]sn[ff]sn[ff]s"
        }

        self.data['ranges']['BS0010'] = {
            0.520:          "[64]@[40]sn[65]sn[82]sn[6c]s",
            1.1 :           "[64]@[14]sn[61]sn[6f]sn[70]s",
            3.5 :           "[64]@[5c]sn[50]sn[3a]sn[81]s",
            5.2 :           "[64]@[9d]sn[44]sn[38]sn[8d]s",
            11 :            "[64]@[28]sn[1c]sn[c1]sn[b5]s",
            "max" :         "[64]@[00]sn[00]sn[ff]sn[ff]s"
        }

        """STATE. IT'S HERE"""
        self.state = self.s_find_device

        """Measurement data"""
        self.data['ch'] = {}
        self.data['ch']['a'] = {
            'raw_avg':  0,
            'clip':     {'top':False, 'bot':False},
            'ana':      0.0,
            'display':  0.0,
            'ready':    {'data':False, 'display':False},
            'ena':      False,
            'was_ena':  False,
            'text_out': ""
        }

        """Mult"""
        self.data['mult'] = {'values':[1,2,5,10]}
        self.data['ch']['a']['mult_index'] = 0

        """Logic"""
        self.data['logic'] = {
            'step':     0.100,
            'rounding': 3,
            'min':      -6.000,
            'max':      6.000
        }
        self.data['ch']['a']['logic'] = {'top':1.5,'bot':0.0,'ena':False,'val':0}
        
        """OTHER"""
        self.data['range'] = 11
        self.data['range_changed'] = False

        self.data['ch']['a']['zero'] = 0.00

        self.data['diff'] = {
            'ena': False,
            'zero':0.00,
            'display':0.00,
            'text_out':""
        }

        self.data['frame_toggle'] = 0

        """Interpolation for Enhanced"""
        self.data['enhanced'] = {
            'ena': False,
            'frame': {'count': 0, 'max': 24}
        }
        self.data['ch']['a']['interp'] = {
            'queue': [0,0,0,0,0,0,0,0,0],
            'value': 0.0
        }

        self.data['device'] = {
            'model':None,
            'connected':False
        }
        
        self.data['mode'] = {
            'single':True,
            'dual':False,
            'diff':False,
            'logic':False
        }

        self.data['ch']['b'] = {}
        copy_dict(self.data['ch']['a'], self.data['ch']['b'])
        
        self.a = self.data['ch']['a']
        self.b = self.data['ch']['b']
        
        # Start the engine.... In the key of a.
        self.change_mode('single')
        

    """User callable functions"""
    def step_limit(self, ch, top_bot, inc):
        if self.data['ch'][ch]['logic']['ena']:
            lim = self.data['ch'][ch]['logic']
            lim[top_bot] = round(lim[top_bot] + inc, 1)
            if lim[top_bot] < self.data["logic"]['min']:
                lim[top_bot] = self.data["logic"]['min']
            elif lim[top_bot] > self.data["logic"]['max']:
                lim[top_bot] = self.data["logic"]['max']

    def toggle_limits(self, ch):
        self.data['ch'][ch]['logic']['ena'] = not self.data['ch'][ch]['logic']['ena']

    def step_mult(self, ch, inc):
        if self.data['ch'][ch]['ena']:
            ch = self.data['ch'][ch]
            ch['mult_index'] += inc
            if ch['mult_index'] < 0:
                ch['mult_index'] = 0
            elif ch['mult_index'] >= len(self.data['mult']['values']):
                ch['mult_index'] = len(self.data['mult']['values']) - 1

    def toggle_ch(self, ch):
        self.data['ch'][ch]['ena'] = not self.data['ch'][ch]['ena']

    def force_ch(self, ch, state):
        self.data['ch'][ch]['ena'] = state
        
    def change_mode(self, mode):
        if mode == "single":
            self.force_ch('a', True)
            self.force_ch('b', False)
            self.force_diff(False)
        elif mode == "dual":
            self.force_ch('a', True)
            self.force_ch('b', True)
            self.force_diff(False)
        elif mode == "diff":
            self.force_ch('a', True)
            self.force_ch('b', True)
            self.force_diff(True)
        elif mode == "logic":
            self.force_ch('a', True)
            self.force_ch('b', True)
            self.force_diff(False)
            
        for m in self.data['mode'].keys():
            if m == mode:
                self.data['mode'][m] = True
            else:
                self.data['mode'][m] = False

    def zero_ch(self, ch):
        if self.data['ch'][ch]['ena']:
            if self.data['diff']['ena'] and ch == 'a':
                self.data['diff']['zero'] = self.data['ch'][ch]['display'] + self.data['diff']['zero']
            else:
                self.data['ch'][ch]['zero'] = self.data['ch'][ch]['display'] + self.data['ch'][ch]['zero']

    def toggle_diff(self):
        self.data['diff']['ena'] = not self.data['diff']['ena']

    def force_diff(self, state):
        self.data['diff']['ena'] = state

    def toggle_sound(self):
        self.sound_enabled = not self.sound_enabled

    def toggle_enhanced(self):
        self.data['enhanced']['ena'] = not self.data['enhanced']['ena']

    def force_enhanced(self, state):
        self.data['enhanced']['ena'] = state

    """STATES"""

    def s_find_device(self):
        self.ser = self.s_t.find_device()
        if self.ser != None:
            self.data['device']['connected'] = True
            self.state = self.s_check_model
        else:
            if self.device_connected:
                self.data['device']['connected'] = False
            self.state = self.s_find_device

    def s_check_model(self):
        self.ser.read(10000) #Try to get anything in the buffer.
        self.s_t.clear_waiting() #Force the counter to reset.
        self.s_t.issue_wait("?")
        self.data['device']['model'] = (self.ser.read(20)[1:7])
        print self.data['device']['model'] + " Connected."

        self.dirty = True
        if self.data['device']['model'] in self.data['ranges']:
            self.state = self.s_setup_bs
        else:
            self.state = self.s_check_model

    def s_setup_bs(self): #setup BS.
        self.s_t.issue("[1c]@[%s]sn[%s]s" % (l_endian_hexify(self.whole_dump / 2))) #Dump size
        self.s_t.issue("[1e]@[06]s[21]@[12]s") #set dump and trace mode to filter, and macro
        self.s_t.issue("[08]@[00]sn[00]sn[00]s") #Set spock address
        self.s_t.issue("[16]@[01]sn[00]s") #Set iterations to 1
        
        self.s_t.issue("[2a]@[%s]sn[%s]s" % (l_endian_hexify(self.whole_trace))) #post trig cap (was 1024)
        self.s_t.issue("[30]@[00]s") #dump channel (Doesn't do anything?)
        self.s_t.issue("[31]@[04]s") #Buffer mode (to macro)
        self.s_t.issue("[37]@[01]s") #Analogue channel enable
        self.s_t.issue("[26]@[%s]sn[%s]s" % (l_endian_hexify(16))) #pre trig cap
        self.s_t.issue("[2c]@[00]sn[0a]s") #Time out REALLY IMPORTANT
        self.s_t.issue("[2e]@[90]sn[01]s") #Set clock ticks
        self.s_t.issue("[14]@[01]sn[00]s") #clock scale
        
        """TRIGGER"""
        self.s_t.issue("[06]@[00]s") #Set trigger mask to "Don't care about anything"
        self.s_t.issue("[05]@[00]s") #This doesn't matter because we don't care about triggers.

        self.s_t.issue("[28]@[00]sn[00]s") #pre trigger delay
        self.s_t.issue("[32]@[00]sn[00]sn[00]sn[00]s") #Edge trigger intro, then outro filter.
        self.s_t.issue("[44]@[00]sn[00]s") #dig comparitor trigger (signed)
        self.s_t.issue("[40]@[00]sn[00]sn[00]sn[00]s") #stop watch trigger (ticks)

        """Filter mode stuff"""
        self.s_t.issue("[18]@[10]sn[00]s") #Dump send to 16
        self.s_t.issue("[1a]@[00]sn[00]s") #skip = 0 (sum 16, skip 0, sum 16, skip 0, ...)

        self.s_t.issue(self.data['ranges'][self.data['device']['model']][self.data['range']]) #Range entry
        self.ser.flushInput()

        self.s_t.issue_wait(">")
        self.s_t.issue("U")
        
        self.state = self.s_change_to_chop

    def s_change_to_chop(self):
        self.ser.flushInput()
        self.s_t.clear_waiting()
        self.s_t.issue("[1e]@[06]s") #Set dump mode stays as filter
        self.s_t.issue("[21]@[13]s") #Set trace mode (macro chop)
        self.s_t.issue("[31]@[05]s") #Buffer mode
        self.s_t.issue("[37]@[03]s") #Ana ch enable

        self.s_t.issue_wait(">")
        self.s_t.issue("U")

        self.state = self.s_pre_idle

    def s_pre_idle(self):
        self.s_t.clear_waiting()
        self.ser.flushInput()
        self.state = self.s_idle

    def s_idle(self): #Idle
        self.s_t.issue_wait("?")
        self.ser.flushInput()
        if self.a['ena'] or self.b['ena']:
            self.state = self.s_init_a_b_req_chop

    """Dual chop"""
    def s_init_a_b_req_chop(self):
        self.s_t.clear_waiting()
        self.ser.flushInput()
        self.s_t.issue_wait(">")
        self.s_t.issue("[37]@[03]sD")
        self.state = self.s_a_dump_chop

    def s_a_dump_chop(self):
        self.s_t.clear_waiting()
        self.ser.read(33)
        self.s_t.issue_wait(">")
        self.s_t.issue("[30]@[00]sA") #Dump chA

        if self.a['ena'] or self.b['ena']:
            self.state = self.s_a_proc_b_dump_chop
        else:
            self.state = self.s_pre_idle

    def s_a_proc_b_dump_chop(self):
        self.s_t.clear_waiting()
        if self.data['ch']['a']['ena']:
            self.ser.read(self.ex_size)
            avg, clips = avg_dump_12bit(self.ser.read(self.d_size))
            self.a['raw_avg'] = avg
            self.a['clip']['top'] = clips[0]
            self.a['clip']['bot'] = clips[1]
            self.a['ready']['data'] = True
        else:
            self.ser.read(self.d_size + self.ex_size) #Read off dump as junk.
            self.b['ready']['data'] = False
        self.s_t.issue_wait(">")
        self.s_t.issue("[30]@[01]sA") #Dump chB

        self.state = self.s_b_proc_a_b_req_chop

    def s_b_proc_a_b_req_chop(self):
        self.s_t.clear_waiting()
        if self.b['ena']:
            self.ser.read(self.ex_size)
            avg, clips = avg_dump_12bit(self.ser.read(self.d_size))
            self.b['raw_avg'] = avg
            self.b['clip']['top'] = clips[0]
            self.b['clip']['bot'] = clips[1]

            self.b['ready']['data'] = True
        else:
            self.ser.read(self.d_size + self.ex_size) #Read off dump as junk.
        self.a['ready']['data'] = False
        self.s_t.issue_wait(">")
        self.s_t.issue("[37]@[03]sD") #Request both channels.

        self.state = self.s_a_dump_chop

    """Utility functions"""
    def volt_interp(self, ls, new_v):
        ls.pop(0)
        ls.append(new_v)
        return round((sum(ls) / len(ls)), 4)

    def update(self):
        """Do state!"""
        try:
            self.state()
        except serial.SerialException:
            print "Device disconected | Error: SE"
            self.state = self.s_find_device
        except serial.SerialTimeoutException:
            print "Device disconected | Error: STE"
            self.state = self.s_find_device

        a = self.data['ch']['a']
        b = self.data['ch']['b']

        """data post-process!"""
        if self.data['device']['connected']:            
            if a['ena']:
                if a['ready']['data']:
                    a['ana'] = convert_to_volt_12bit(a['raw_avg'], self.data['range'])
                    a['ana'] = round((a['ana'] - a['zero']), 6) #zero
                    a['display'] = a['ana'] * self.data['mult']['values'][a['mult_index']] # apply multiplier
                    if a['logic']['ena']:
                        if a['display'] >= a['logic']['top']:
                            log = 1
                        elif a['display'] <= a['logic']['bot']:
                            log = 2
                        else:
                            log = 0
                    else:
                        log = 0
                    a['logic']['val'] = log

            if b['ena']:
                if b['ready']['data']:
                    b['ana'] = convert_to_volt_12bit(b['raw_avg'], self.data['range'])
                    b['ana'] = round((b['ana'] - b['zero']), 6) #zero
                    b['display'] = b['ana'] * self.data['mult']['values'][b['mult_index']] # apply multiplier
                    if b['logic']['ena']:
                        if b['display'] >= b['logic']['top']:
                            log = 1
                        elif b['display'] <= b['logic']['bot']:
                            log = 2
                        else:
                            log = 0
                    else:
                        log = 0
                    b['logic']['val'] = log
                
            """INTERP MODE"""
            if self.data['enhanced']['ena']:
                if a['ena'] and a['ready']['data']:
                    a['interp']['value'] = self.volt_interp(a['interp']['queue'], a['display'])
                    a['display'] = a['interp']['value']
                    
                if b['ena'] and b['ready']['data']:
                    b['interp']['value'] = self.volt_interp(b['interp']['queue'], b['display'])
                    b['display'] = b['interp']['value']

            """DIFF MODE"""
            if self.data['diff']['ena']:
                a['display'] = a['ana'] - b['ana']
                a['display'] = a['display'] - self.data['diff']['zero']

            """SLOW DOWN INTERP"""
            if self.data['enhanced']['ena']:
                if a['ena'] and self.data['enhanced']['frame']['count'] == 0:
                    a['ready']['display'] = True
                if b['ena'] and self.data['enhanced']['frame']['count'] == 12:
                    b['ready']['display'] = True
                
                self.data['enhanced']['frame']['count'] += 1
                if self.data['enhanced']['frame']['count'] >= self.data['enhanced']['frame']['max']-1:
                    self.data['enhanced']['frame']['count'] = 0

            else:
                a['ready']['display'] = True
                b['ready']['display'] = True

            if self.data['enhanced']['ena']: #check interp, set rounding
                amount = "4"
            else:
                amount = "3"
                
            """Set the text!"""
            reduce_to = "%." + amount + "f" #max length of voltage is 8 chars
            if a['ena'] and a['ready']['display']:
                a['text_out'] = (reduce_to % a['display'] + "v").rjust(8)
                a['ready']['display'] = False
            if b['ena'] and b['ready']['display']:
                b['text_out'] = (reduce_to % b['display'] + "v").rjust(8)
                b['ready']['display'] = False
